@@ -150,6 +150,7 @@ Application::Application(Application::OperationMode mode, int &argc, char **argv
     : QApplication(argc, argv)
     , m_eventFilter(new XcbEventFilter())
     , m_configLock(false)
+    , m_config()
     , m_operationMode(mode)
 {
     qRegisterMetaType<Options::WindowOperation>("Options::WindowOperation");
@@ -184,11 +185,13 @@ void Application::start()
 {
     setQuitOnLastWindowClosed(false);
 
-    KSharedConfig::Ptr config = KSharedConfig::openConfig();
-    if (!config->isImmutable() && m_configLock) {
+    if (!m_config) {
+        m_config = KSharedConfig::openConfig();
+    }
+    if (!m_config->isImmutable() && m_configLock) {
         // TODO: This shouldn't be necessary
         //config->setReadOnly( true );
-        config->reparseConfiguration();
+        m_config->reparseConfiguration();
     }
 
     crashChecking();
@@ -208,9 +211,14 @@ void Application::destroyAtoms()
     atoms = nullptr;
 }
 
-void Application::crashChecking()
+void Application::setupCrashHandler()
 {
     KCrash::setEmergencySaveFunction(Application::crashHandler);
+}
+
+void Application::crashChecking()
+{
+    setupCrashHandler();
     if (crashes >= 4) {
         // Something has gone seriously wrong
         AlternativeWMDialog dialog;
@@ -239,13 +247,6 @@ void Application::crashChecking()
     QTimer::singleShot(15 * 1000, this, SLOT(resetCrashesCount()));
 }
 
-bool Application::notify(QObject* o, QEvent* e)
-{
-    if (Workspace::self()->workspaceEvent(e))
-        return true;
-    return QApplication::notify(o, e);
-}
-
 void Application::crashHandler(int signal)
 {
     crashes++;
@@ -529,4 +530,3 @@ QProcessEnvironment Application::processStartupEnvironment() const
 
 } // namespace
 
-#include "main.moc"
